/*
   Projet de simulation d'inondation.
   
   Version C++ utilisant QT et OpenGL.
   
   10 Avril 2004

   (C) Eric Beaudry & Universit de Sherbrooke 2004.
*/


#include <qgl.h>
#include <qevent.h>
#include <qimage.h>
#include "util.h"
#include "terrain.h"
#include "propag.h"

class Mesh{
   public:
     Mesh(int w, int h);
     ~Mesh();
     
     Point&   getSommet(int x, int y) { return sommets[x + y * width];}
     const Vecteur& getVN(int x, int y) const { return vecteurs[x + y * width];}
     
     void autoVN(); // calcul tous les VN
     
     int  getWidth() const { return width;}
     int  getHeight() const { return height;}
   
   protected:
     int         width;
     int         height;
     
     Point*      sommets;
     Vecteur*    vecteurs; // vecteurs normaux
};


class SimInondWidget : public QGLWidget {
public:
    SimInondWidget( Terrain* terrain, QWidget *parent=0, const char *name=0 );
    ~SimInondWidget();
    
    
    void setTerrain(Terrain* terrain);
    void updateMeshEau();
    void rotation(float angle);
    
    void setTexture(const QImage& image);
    
protected:
    void initializeGL();
    void resizeGL( int, int );
    void paintGL();
    void timerEvent( QTimerEvent * );
    
    void keyPressEvent(QKeyEvent* e);
    void keyReleaseEvent(QKeyEvent* e);
    
    void mouseMoveEvent(QMouseEvent* e);
    void mousePressEvent(QMouseEvent* e);
    
    QPoint      lastPoint;
    bool        shiftDown;
    
    QImage      texture;
    
    Mesh*       meshSol;
    Mesh*       meshEau;
    Terrain*    terrain;
    float       minTerrain, maxTerrain;
    float       angleRotation;
    float       angleSol;
    float       refX, refY;
    float       distanceCamera;
};

