/*
   Projet de simulation d'inondation.
   
   Version C++ utilisant QT et OpenGL.
   
   10 Avril 2004

   (C) Eric Beaudry & Universit de Sherbrooke 2004.
*/

#include "main.h"
#include <assert.h>
#include <qapp.h>
#include <qmainwindow.h>
#include <qaccel.h>
#include <qaction.h>
#include <qcolor.h> 
#include <qcheckbox.h>
#include <qimage.h>
#include <qlcdnumber.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qslider.h>
#include <qspinbox.h>
#include "controlbase.h"
#include "loaddialog.h"


SimInondMainWindow::SimInondMainWindow(){
  setCaption("Estritel - Simulation Inondation avec MNT");
  //terrain = Terrain::generateRandom(101, 101, 0, 20); //new Terrain(131, 131);
  //terrain->lisser();
  terrain = Terrain::generateSinSin(101, 101);
  modelsimple = new ModeleSimple(0.25);
  terrainView = new SimInondWidget(terrain, this);
  setCentralWidget(terrainView);
  
  createMenus();
  
  QDockWindow* dw = new QDockWindow( QDockWindow::InDock, this );
  controlWidget = new ControlBase(dw);
  dw->setResizeEnabled( TRUE );
  dw->setCloseMode( QDockWindow::Always );
  addDockWindow( dw, DockBottom, true);
  dw->setWidget( controlWidget );
  dw->setCaption("Control");
  
  numeroIteration = 0;
  startTimer( 25 );
}

void SimInondMainWindow::createMenus(){
  QPopupMenu* fileMenu = new QPopupMenu();
  menuBar()->insertItem(tr("&File"), fileMenu);
  
  QAction* fileLoadMnt = new QAction(tr("Charger MNT"), tr("&Charger MNT"), 0, this);
  fileLoadMnt->addTo(fileMenu);
  connect(fileLoadMnt, SIGNAL(activated()), this, SLOT(slotLoadMnt()));
   
  fileMenu->insertSeparator();
  QAction* fileQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
  fileQuit->addTo(fileMenu);
  connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));
  
  QPopupMenu* testMenu = new QPopupMenu();
  menuBar()->insertItem(tr("&Test"), testMenu);
  
  QAction* testSin = new QAction(tr("Mesh Courbes Sin Sin"), 0, this);
  connect(testSin, SIGNAL(activated()), this, SLOT(slotTestSin()));
  testSin->addTo(testMenu);
  
  QAction* testRandom = new QAction(tr("Random Montagnes / Lacs"), 0, this);
  connect(testRandom, SIGNAL(activated()), this, SLOT(slotTestRandom()));
  testRandom->addTo(testMenu);
  
}

void SimInondMainWindow::timerEvent(QTimerEvent*)
{
   int p = controlWidget->sliderPluie->value();
   bool etatchanged = false;
   if(controlWidget->simButton->state() == QButton::On){
      if(p!=0){
        double pluie = p / 1000.0 / 100; // pluie en mtre / 100 itration
        for(int x=1;x<terrain->getWidth()-1;x++)
          for(int y=1;y<terrain->getHeight()-1;y++){
            terrain->eau(x, y) += pluie;
	    if(terrain->eau(x, y)<0) terrain->eau(x, y) = 0;
          }
        etatchanged = true;
      }
      if(controlWidget->checkBoxActif->isChecked()){
         modelsimple->setRate(controlWidget->spinBoxTaux->value() / 100.0);
	 modelsimple->step(terrain);
         etatchanged = true;
      }
      numeroIteration++;
      controlWidget->lCDNumberIter->display(numeroIteration);
   }
   
   int r = controlWidget->spinBoxAutoRotation->value();
   terrainView->rotation(r);
   
   if(etatchanged)
      terrainView->updateMeshEau();
   
   if(r!=0 || etatchanged)
      terrainView->updateGL();
}

void SimInondMainWindow::slotLoadMnt(){
   LoadDialog ld(this);
   if(ld.exec() == QDialog::Accepted){
      QImage image;
      if(!image.load(ld.fileMntEdit->text())){
         QMessageBox::critical(this, "Chargement Terrain", "Erreur ouverture image MNT");
         return;
      }
      
      float resZ = ld.resZEdit->text().toFloat();
      delete terrain;
      
      terrain = new Terrain(image.width(), image.height());
      for(int x=0;x<image.width();x++)
        for(int y=0;y<image.height();y++){
	  terrain->sol(x, y) = resZ * qGray(image.pixel(x, y));
	  //assert(terrain->eau(x, y) == 0);
	}
	  
      terrainView->setTerrain(terrain);
      
      QImage texture;
      if(texture.load(ld.textureEdit->text())){
         terrainView->setTexture(texture);
      }
   }
}

void SimInondMainWindow::slotFileQuit()
{
  qApp->quit();
}

void SimInondMainWindow::slotTestSin(){
   delete terrain;
   terrain = Terrain::generateSinSin(151, 151);
   terrainView->setTerrain(terrain);
}

void SimInondMainWindow::slotTestRandom(){
   delete terrain;
   terrain = Terrain::generateRandom(151, 151, 7, 8);
   terrainView->setTerrain(terrain);
}

int main( int argc, char **argv )
{
    QApplication::setColorSpec( QApplication::CustomColor );
    QApplication a( argc, argv );

    if ( !QGLFormat::hasOpenGL() ) {
	qWarning( "This system has no OpenGL support. Exiting." );
	return -1;
    }

    SimInondMainWindow mw;
    a.setMainWidget( &mw );
    mw.resize(800, 600);
    mw.show();
    return a.exec();
}

