/*
   Projet de simulation d'inondation.
   
   Version C++ utilisant QT et OpenGL.
   
   10 Avril 2004

   (C) Eric Beaudry & Universit de Sherbrooke 2004.
*/

#include "propag.h"
#include <math.h>
#include <iostream>
#include <stdlib.h>
#include <assert.h>

ModeleSimple::ModeleSimple(double r) : rate(r), delta(0) {
 
}

ModeleSimple::~ModeleSimple(){
   delete[] delta;
}

void ModeleSimple::setRate(double r) { 
   rate = r;
}

void ModeleSimple::step(Terrain* terrain){
   if(delta==0 || width!=terrain->getWidth() || height!=terrain->getHeight()){
      delete[] delta;
      width = terrain->getWidth();
      height = terrain->getHeight();
      delta = new double[width * height];
   }
   
   // reset les delta  zro...
   for(int i=0;i<width*height;i++) delta[i] = 0.0;
   
   static int dx[] = { -1, 0, 1, 1, 1, 0, -1, -1};
   static int dy[] = { -1, -1, -1, 0, 1, 1, 1, 0};
   
   // Les bords ne seront pas traits
   for(int y=1;y<height-1;y++)
      for(int x=1;x<width-1;x++){
          // quantit d'eau dans la cellule centrale
          double qteCentral = terrain->eau(x, y);
	  if(qteCentral<=0) break;
	  double totCentral = terrain->total(x, y);
	  double totaldiff = 0;
	  double diff[8];
	  double minimum = totCentral;
	  
	  for(int v=0;v<8;v++){
	     double t = terrain->total(x+dx[v], y+dy[v]);
	     if(t<minimum) minimum = t;
	     diff[v] = totCentral - t;
	     if(diff[v] < 0)
	        diff[v] = 0;
	       else
	        totaldiff += diff[v];
	  }
	  
	  if(totaldiff > 0){
	      double redist = (qteCentral < totaldiff ? qteCentral : totaldiff);
	      if(redist > totCentral - minimum) redist = totCentral-minimum;
	      redist *= rate;
	      delta[x+y*width] -= redist;
	      
	      for(int v=0;v<8;v++)
	         delta[x+dx[v] + (y+dy[v])*width] += diff[v] * redist / totaldiff;
	  }
      }

   for(int y=0;y<height;y++)
      for(int x=0;x<width;x++)
         terrain->eau(x, y) += delta[x+y*width];
}


