/*
   Projet de simulation d'inondation.
   
   Version C++ utilisant QT et OpenGL.
   
   10 Avril 2004

   (C) Eric Beaudry & Universit de Sherbrooke 2004.
*/


#include "terrain.h"
#include <iostream>
#include <stdlib.h>
#include <math.h>
#include <time.h>

Terrain::Terrain(int w, int h, double niveau) : width(w), height(h){
   resolution = 1;
   ep_eau = new double[w*h];
   nivsol = new double[w*h];
   for(int i=0;i<w*h;i++){
      ep_eau[i] = 0;
      nivsol[i] = niveau;
   }
}

Terrain::~Terrain(){
   delete[] nivsol;
   delete[] ep_eau;
}

void Terrain::lisser(){
   double* nouveau = new double[width*height];
   for(int x=0;x<width;x++)
    for(int y=0;y<height;y++){
       double t = 0;
       int c=0;
       for(int x2=x-1;x2<=x-1;x2++)
         for(int y2=y-1;y2<=y-1;y2++)
	    if(x2>=0 && y2>=0 && x2<width && y2<height){
	      t += sol(x2, y2);
	      c++;
	    }
       nouveau[x + y*width] = t/c;
    }
   delete[] nivsol;
   nivsol = nouveau;
}

void  Terrain::approcherVoisin(int x, int y, double cible, double maxdiff){
   double diff = sol(x, y) - cible;
   if(diff > maxdiff){
      sol(x, y) = cible + maxdiff;
   }else if(diff < -maxdiff){
      sol(x, y) = cible - maxdiff;
   }else return;
   
   if(x>0) approcherVoisin(x-1, y, sol(x, y), maxdiff);
   if(x<width-1) approcherVoisin(x+1, y, sol(x, y), maxdiff);
   if(y>0) approcherVoisin(x, y-1, sol(x, y), maxdiff);
   if(y<height-1) approcherVoisin(x, y+1, sol(x, y), maxdiff);
   
}

void  Terrain::getMinMax(double& min, double& max) const{
   int n = width * height;
   min = max = nivsol[0];
   for(int i=1;i<n;i++){
      if(nivsol[n] < min) min = nivsol[n];
      else if(nivsol[n] > max) max = nivsol[n];
   }
}

Terrain* Terrain::generateSinSin(int w, int h){
   Terrain* terrain = new Terrain(w, h, 3);
   for(int x=0;x<w;x++)
    for(int y=0;y<h;y++){
       float xp = x-w/2;
       float yp = y-h/2;
       float zp = 3 + sin(xp/4) + sin(yp/4) + (0.25 * rand() / (RAND_MAX+1));
       terrain->sol(x,y) = zp;
    }
   return terrain;
}

Terrain* Terrain::generateRandom(int w, int h, int nbmont, int nblac){
   if(w<20) w=20;
   if(h<20) h=20;
   Terrain* terrain = new Terrain(w, h);
   for(int x=0;x<w;x++){
     terrain->approcherVoisin(x, 0, 5, 0.5);
     terrain->approcherVoisin(x, h-1, 5, 0.5);
   }
   for(int y=0;y<h;y++){
     terrain->approcherVoisin(0, y, 5, 0.5);
     terrain->approcherVoisin(w-1, y, 5, 0.5);
   }
   
   time_t t = time(NULL);
   srand(t);
   
   // Gnrer des montagnes
   for(int i=0;i<nbmont + nblac;i++){
      int x1 = (int) (  (w-20.0) * rand() / (RAND_MAX+1.0)) + 10;
      int x2 = (int) (  10.0 * rand() / (RAND_MAX+1.0)) + x1;
      int y1 = (int) (  (h-20.0) * rand() / (RAND_MAX+1.0)) + 10;
      int y2 = (int) (  10.0 * rand() / (RAND_MAX+1.0)) + y1;
      
      int sens = i<nbmont ? 1 : -1; 
      double hauteur = 5.0 * rand() / (RAND_MAX+1.0);
      if(i>nbmont)
         hauteur *= -1;
	 
      hauteur += terrain->sol((x1+x2)/2, (y1+y2)/2);
      
      for(int x=x1;x<=x2;x++)
        for(int y=y1;y<y2;y++){
	   double dx = (x - x1) * 2.0 / (x2+x1) - 1.0;
	   double dy = (x - x1) * 2.0 / (x2+x1) - 1.0;
	   double he = (1 - dx*dx + dy*dy) * sens;
	   terrain->approcherVoisin(x, y, hauteur+he, 0.25);
	   terrain->sol(x, y) = hauteur+he;
	}
   }
   return terrain;
}

