/*
   Projet de simulation d'inondation.
   
   Version C++ utilisant QT et OpenGL.
   
   10 Avril 2004

   (C) Eric Beaudry & Universit de Sherbrooke 2004.
*/


#include <math.h>
#include "util.h"

Point::Point(float x_, float y_, float z_)
{
	this->x = x_;
	this->y = y_;
	this->z = z_;
}

Vecteur Point::operator - (const Point& p) const{
        return Vecteur(p.x-x, p.y-y, p.z-z);
}

Vecteur::Vecteur(float x_, float y_, float z_)
{
	this->x = x_;
	this->y = y_;
	this->z = z_;
}

Vecteur::Vecteur(const Point& p1, const Point& p2){
	x = p2.x - p1.x;
	y = p2.y - p1.y;
	z = p2.z - p1.z;
}

Vecteur& Vecteur::operator += (const Vecteur& autre){
	x += autre.x;
	y += autre.y;
	z += autre.z;
	return *this;
}

Vecteur  Vecteur::operator +  (const Vecteur& autre) const{
	Vecteur r(*this);
	r += autre;
	return r;
}

Vecteur& Vecteur::operator *= (const float& scale){
	x *= scale;
	y *= scale;
	z *= scale;
	return *this;
}

// Produit vectoriel (trouver vecteur normal)    
Vecteur  Vecteur::operator ^  (const Vecteur& a) const{
	return Vecteur(y*a.z - a.y*z,
		       z*a.x - a.z*x,
		       x*a.y - a.x*y);
}

void Vecteur::normalise(){
   float n = norme();
   x /= n;
   y /= n;
   z /= n;
}

float Vecteur::norme2(){
   return x*x + y*y + z*z;
}

float Vecteur::norme(){
   return sqrt(x*x + y*y + z*z);
}



